/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sem;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import sem.model.Zacatek;
import sem.model.Osoba;
import sem.model.Konec;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.Locale;
import sem.model.Lokalita;
import sem.model.Turnus;
import java.util.ArrayList;
import java.util.Collection;
import sem.model.Skupina;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import javax.xml.stream.Location;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.XMLEvent;
import sem.model.CisloSkupiny;
import sem.model.Dite;
import sem.model.Jmeno;
import sem.model.Pobyty;
import sem.model.PobytyElement;
import sem.model.Pohlavi;
import sem.model.PohlaviEnum;
import sem.model.Prijmeni;
import sem.model.RokNarozeni;
import sem.model.Vedouci;
import static sem.model.PobytyElement.*;

/**
 *
 * @author Radka
 */
public class PobytyParser {

    private XMLEventReader reader;

    PobytyParser(String source) throws FileNotFoundException, XMLStreamException {
        InputStream sourceInputStream = new FileInputStream(source);
        XMLInputFactory factory = XMLInputFactory.newInstance();
        reader = factory.createXMLEventReader(sourceInputStream);
        assert reader != null;
    }

    //pomocne PRIVATE metody

    private void error(String message, XMLEvent event) throws SyntaxeErrorEvent {
        throw new SyntaxeErrorEvent(message, event);
    }   

    //vrati aktualni element, ktery je na rade
    private XMLEvent currElement()
            throws XMLStreamException {

        return reader.peek();
    }

    //skip element
    private void skipElement() throws XMLStreamException {
        reader.nextEvent();
    }

    //test startovniho elementu - je start?, jakeho je druhu?
    boolean isCurrentStartElement(PobytyElement tagName) throws XMLStreamException {
        skipWhiteSpaceElement();
        if (!currElement().isStartElement()) {
            return false;
        }
        String localPart = currElement().asStartElement().getName().getLocalPart();
        return tagName.toString().equals(localPart);    //porovnani, zda je start element shodny
        //s pozadovanym prvkem tagName
    }

    private void skipStartElement(PobytyElement tagName) throws XMLStreamException, SyntaxeErrorEvent {
        skipWhiteSpaceElement();
        checkStartElement(tagName);
        skipElement();
    }

    //osetreni koncu radek
    private void skipWhiteSpaceElement() throws XMLStreamException {
        XMLEvent ce = currElement();
        while (!ce.isStartElement() && !ce.isEndElement() && !ce.isEndDocument()) {
            skipElement();
            ce = currElement();
        }

    }

    private void skipEndElement(PobytyElement tagName) throws SyntaxeError {
        try {
            skipWhiteSpaceElement();
            XMLEvent event = currElement();
            if (!event.isEndElement()) {
                error("Expected end element: " + tagName, event);
            }

            EndElement endElement = event.asEndElement();
            if (!endElement.getName().getLocalPart().equals(tagName.toString())) {
                error("Expected end element: " + tagName, event);
            }
            skipElement();
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    private String nextText() throws XMLStreamException {
        String text = reader.getElementText();
        return text;
    }

    private void checkStartElement(PobytyElement tagName) throws XMLStreamException, SyntaxeErrorEvent {
        if (!isCurrentStartElement(tagName)) {
            throw new SyntaxeErrorEvent("Start element " + tagName + " expected", currElement());
        }
    }

    //--------------------------------------------------------------------------

    public Dite parseDite() throws SyntaxeError {
        try {
            skipStartElement(dite);
            Osoba osoba = parseOsoba();
            Pohlavi pohlavi = parsePohlavi();
            skipEndElement(dite);
            return new Dite(osoba, pohlavi);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Osoba parseOsoba() throws SyntaxeError {
        Jmeno jmeno = parseJmeno();
        Prijmeni prijmeni = parsePrijmeni();
        RokNarozeni rokNarozeni = parseRokNarozeni();
        return new Osoba(jmeno, prijmeni, rokNarozeni);
    }

    public Vedouci parseVedouci() throws SyntaxeError {
        try {
            skipStartElement(vedouci);
            Osoba osoba = parseOsoba();
            skipEndElement(vedouci);
            return new Vedouci(osoba);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    Skupina parseSkupina() throws SyntaxeError {
        try {
            skipStartElement(skupina);      
            CisloSkupiny cisloSkupiny = parseCisloSkupiny();
            Vedouci vedouci = parseVedouci();
            Collection<Dite> deti = parseDeti();
            skipEndElement(skupina);
            return new Skupina(cisloSkupiny, vedouci, deti);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }

    }

    public List<Dite> parseDeti() throws SyntaxeError {
        try {
            if (isCurrentStartElement(dite)) {
                Dite d = parseDite();
                List<Dite> di = parseDeti();
                di.add(0, d);
                return di;
            } else {
                ArrayList<Dite> deti = new ArrayList<Dite>();
                return deti;
            }
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Pohlavi parsePohlavi() throws SyntaxeError {
        XMLEvent elem = null;
        try {
            skipStartElement(pohlavi);
            String pohlavi = nextText();
            elem = currElement();
            return new Pohlavi(PohlaviEnum.valueOf(pohlavi));
        } catch (IllegalArgumentException ex) {
            throw new SyntaxeErrorEvent(ex, elem);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public RokNarozeni parseRokNarozeni() throws SyntaxeError {
        XMLEvent elem = null;
        try {
            skipStartElement(rokNarozeni);
            String rok = nextText();
            elem = currElement();
            return new RokNarozeni(Integer.parseInt(rok));
        } catch (NumberFormatException ex) {
            throw new SyntaxeErrorEvent(ex, elem);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public CisloSkupiny parseCisloSkupiny() throws SyntaxeError {
        Location l = null;
        try {
            skipStartElement(cisloSkupiny);
            String cislo = nextText();
            l = currElement().getLocation();
            return new CisloSkupiny(Integer.parseInt(cislo));
        } catch (NumberFormatException ex) {
            throw new SyntaxeError(ex, l);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Jmeno parseJmeno() throws SyntaxeError {
        try {
            skipStartElement(jmeno);
            String strJmeno = nextText();
            return new Jmeno(strJmeno);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Prijmeni parsePrijmeni() throws SyntaxeError {
        try {
            skipStartElement(prijmeni);
            String strPrijmeni = nextText();
            return new Prijmeni(strPrijmeni);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Lokalita parseLokalita() throws SyntaxeError {
        try {
            skipStartElement(lokalita);
            String strLoklalita = nextText();
            return new Lokalita(strLoklalita);

        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Konec parseKonec() throws SyntaxeError {
        try {
            skipStartElement(konec);
            return new Konec(parseDatum());
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Date parseDatum() throws SyntaxeError {
        Location l = null;
        try {
            l = currElement().getLocation();
            String strDatum = nextText();
            DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, Locale.getDefault());
            Date datum = df.parse(strDatum);
            return datum;
        } catch (ParseException ex) {
            throw new SyntaxeError(ex, l);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Zacatek parseZacatek() throws SyntaxeError {
        try {
            skipStartElement(zacatek);
            return new Zacatek(parseDatum());
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Pobyty parsePobyty() throws SyntaxeError {
        try {
            skipElement();      //skip StartDokumentu, pridano StAX, neni videt v XML
            skipStartElement(pobyty);
            Collection<Turnus> turnusy = parseTurnusy();
            skipEndElement(pobyty);
            return new Pobyty(turnusy);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    /*
    BEZ REKURZE
    public List<Skupina> parseSkupiny() throws SyntaxeError {
    try {
    ArrayList<Skupina> skupiny = new ArrayList<Skupina>();
    while (isCurrentStartElement(skupina)) {
    skupiny.add(parseSkupina());
    }
    return skupiny;
    } catch (XMLStreamException ex) {
    throw new SyntaxeError(ex);
    }
    }
     */
    
    //S REKURZI
    public List<Skupina> parseSkupiny() throws SyntaxeError {
        Skupina s;
        List<Skupina> sy;
        try {
            if (isCurrentStartElement(skupina)) {
                s = parseSkupina();
                sy = parseSkupiny();
                sy.add(0, s);
                return sy;
            } else {
                ArrayList<Skupina> skupiny = new ArrayList<Skupina>();
                return skupiny;
            }
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    public Turnus parseTurnus() throws SyntaxeError {
        try {
            skipStartElement(turnus);
            Lokalita lokalita = parseLokalita();
            Zacatek zacatek = parseZacatek();
            Konec konec = parseKonec();
            List<Skupina> skupiny = parseSkupiny();
            skipEndElement(turnus);
            return new Turnus(lokalita, zacatek, konec, skupiny);
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }

    /*
    BEZ REKURZE
    public List<Turnus> parseTurnusy() throws SyntaxeError {
    try {
    ArrayList<Turnus> turnusy = new ArrayList<Turnus>();
    while (isCurrentStartElement(turnus)) {
    turnusy.add(parseTurnus());
    }
    return turnusy;
    } catch (XMLStreamException ex) {
    throw new SyntaxeError(ex);
    }
    }
     */

    //S REKURZI
    public List<Turnus> parseTurnusy() throws SyntaxeError {
        Turnus t;
        List<Turnus> ty;
        try {
            if (isCurrentStartElement(turnus)) {
                t = parseTurnus();
                ty = parseTurnusy();
                ty.add(0, t);
                return ty;
            } else {
                ArrayList<Turnus> turnusy = new ArrayList<Turnus>();
                return turnusy;
            }
        } catch (XMLStreamException ex) {
            throw new SyntaxeError(ex);
        }
    }
}
